@@ -31,13 +31,14 @@ public :
     void    Open_Buffer_Init        (File__Analyze* Sub);
     void    Open_Buffer_Init        (File__Analyze* Sub, int64u File_Size);
     void    Open_Buffer_OutOfBand   (                    const int8u* Buffer, size_t Buffer_Size) {File__Analyze::Buffer=Buffer; File__Analyze::Buffer_Size=Buffer_Size; Element_Offset=0; Element_Size=Buffer_Size; Read_Buffer_OutOfBand(); File__Analyze::Buffer=NULL; File__Analyze::Buffer_Size=0; Element_Offset=0; Element_Size=0;}
-    void    Open_Buffer_OutOfBand   (File__Analyze* Sub, const int8u* Buffer, size_t Buffer_Size);
-    void    Open_Buffer_OutOfBand   (File__Analyze* Sub) {if (Element_Offset<=Element_Size) Open_Buffer_OutOfBand(Sub, Buffer+Buffer_Offset+(size_t)Element_Offset, (size_t)(Element_Size-Element_Offset)); Element_Offset=Element_Size;}
+    void    Open_Buffer_OutOfBand   (File__Analyze* Sub                     , size_t Buffer_Size);
+    void    Open_Buffer_OutOfBand   (File__Analyze* Sub) {Open_Buffer_OutOfBand(Sub, Element_Size-Element_Offset);};
     void    Open_Buffer_Continue    (                    const int8u* Buffer, size_t Buffer_Size);
     void    Open_Buffer_Continue    (File__Analyze* Sub, const int8u* Buffer, size_t Buffer_Size, bool IsNewPacket=true, float64 Ratio=1.0);
     void    Open_Buffer_Continue    (File__Analyze* Sub, size_t Buffer_Size) {if (Element_Offset+Buffer_Size<=Element_Size) Open_Buffer_Continue(Sub, Buffer+Buffer_Offset+(size_t)Element_Offset, Buffer_Size); Element_Offset+=Buffer_Size;}
     void    Open_Buffer_Continue    (File__Analyze* Sub) {if (Element_Offset<=Element_Size) Open_Buffer_Continue(Sub, Buffer+Buffer_Offset+(size_t)Element_Offset, (size_t)(Element_Size-Element_Offset)); Element_Offset=Element_Size;}
     void    Open_Buffer_Position_Set(int64u File_Offset);
+    void    Open_Buffer_CheckFileModifications();
     #if MEDIAINFO_SEEK
     size_t  Open_Buffer_Seek        (size_t Method, int64u Value, int64u ID);
     #endif //MEDIAINFO_SEEK
@@ -208,8 +209,9 @@ protected :
 
     //Buffer
     virtual void Read_Buffer_Init ()          {}; //Temp, should be in File__Base caller
-    virtual void Read_Buffer_OutOfBand ()     {}; //Temp, should be in File__Base caller
+    virtual void Read_Buffer_OutOfBand ()     {Open_Buffer_Continue(Buffer, Buffer_Size);} //Temp, should be in File__Base caller
     virtual void Read_Buffer_Continue ()      {}; //Temp, should be in File__Base caller
+    virtual void Read_Buffer_CheckFileModifications() {} //Temp, should be in File__Base caller
     virtual void Read_Buffer_AfterParsing ()  {}; //Temp, should be in File__Base caller
     #if MEDIAINFO_SEEK
     virtual size_t Read_Buffer_Seek (size_t, int64u, int64u); //Temp, should be in File__Base caller
@@ -318,7 +320,9 @@ protected :
     #define Element_Info3(_A,_B,_C)
     #define Element_Info1C(_CONDITION,_A)
     #define Element_Info_From_Milliseconds(_A)
-    #define Element_Parser(_A)
+    #define Element_Parser(_A) {}
+    #define Element_Error(_A) {}
+    #define Param_Error(_A) {}
 
     //Elements - End
     inline void Element_End () {Element_End_Common_Flush();}
@@ -1060,6 +1064,7 @@ public :
         Ztring Parameter;
         Ztring Value;
     };
+    void Fill_SetOptions(stream_t StreamKind, size_t StreamPos, const char* Parameter, const char* Options);
     vector<fill_temp_item> Fill_Temp[Stream_Max+1]; // +1 because Fill_Temp[Stream_Max] is used when StreamKind is unknown
     void Fill_Flush ();
     static size_t Fill_Parameter(stream_t StreamKind, generic StreamPos);
@@ -1369,6 +1374,9 @@ public :
     #else //MEDIAINFO_DEMUX
         #define Demux(_A, _B, _C)
     #endif //MEDIAINFO_DEMUX
+    #if MEDIAINFO_DECODE
+        void Decoded(const int8u* Buffer, size_t Buffer_Size);
+    #endif //MEDIAINFO_DECODE
 
     //Events data
     bool    PES_FirstByte_IsAvailable;
@@ -1387,6 +1395,7 @@ public :
     //Hash
     #if MEDIAINFO_HASH
         HashWrapper*        Hash;
+        int64u              Hash_Offset;
         int64u              Hash_ParseUpTo;
     #endif //MEDIAINFO_HASH
 
