@@ -128,6 +128,9 @@
 #if defined(MEDIAINFO_WM_YES)
     #include "MediaInfo/Multiple/File_Wm.h"
 #endif
+#if defined(MEDIAINFO_WM_YES)
+    #include "MediaInfo/Multiple/File_Wtv.h"
+#endif
 #if defined(MEDIAINFO_XDCAM_YES)
     #include "MediaInfo/Multiple/File_Xdcam_Clip.h"
 #endif
@@ -400,6 +403,7 @@ File__MultipleParsing::File__MultipleParsing()
     #endif
     #if defined(MEDIAINFO_BDAV_YES)
         {File_MpegTs* Temp=new File_MpegTs(); Temp->BDAV_Size=4; Parser.push_back(Temp);}
+        {File_MpegTs* Temp=new File_MpegTs(); Temp->BDAV_Size=4; Temp->NoPatPmt=true; Parser.push_back(Temp);}
     #endif
 //    Only with directories, no By Buffer interface
 //    #if defined(MEDIAINFO_BDMV_YES)
@@ -464,9 +468,11 @@ File__MultipleParsing::File__MultipleParsing()
     #endif
     #if defined(MEDIAINFO_MPEGTS_YES)
         Parser.push_back(new File_MpegTs());
+        {File_MpegTs* Temp=new File_MpegTs(); Temp->NoPatPmt=true; Parser.push_back(Temp);}
     #endif
     #if defined(MEDIAINFO_TSP_YES)
         {File_MpegTs* Temp=new File_MpegTs(); Temp->TSP_Size=16; Parser.push_back(Temp);}
+        {File_MpegTs* Temp=new File_MpegTs(); Temp->TSP_Size=16; Temp->NoPatPmt=true; Parser.push_back(Temp);}
     #endif
     #if defined(MEDIAINFO_MXF_YES)
         Parser.push_back(new File_Mxf());
@@ -504,6 +510,9 @@ File__MultipleParsing::File__MultipleParsing()
     #if defined(MEDIAINFO_WM_YES)
         Parser.push_back(new File_Wm());
     #endif
+    #if defined(MEDIAINFO_WTV_YES)
+        Parser.push_back(new File_Wtv());
+    #endif
     #if defined(MEDIAINFO_XDCAM_YES)
         Parser.push_back(new File_Xdcam_Clip());
     #endif
