@@ -311,7 +311,11 @@ bool File_Jpeg::Demux_UnpacketizeContainer_Test()
     if (!IsSub)
     {
         if (!Status[IsAccepted])
+        {
             Accept();
+            if (Config->Demux_EventWasSent)
+                return false;
+        }
         if (Config->File_Names.size()>1)
             return Demux_UnpacketizeContainer_Test_OneFramePerFile();
     }
