@@ -90,6 +90,7 @@ protected :
     //Buffer - Global
     void Read_Buffer_Init ();
     void Read_Buffer_Continue ();
+    void Read_Buffer_CheckFileModifications();
     void Read_Buffer_AfterParsing ();
     void Read_Buffer_Unsynched();
     #if MEDIAINFO_SEEK
@@ -172,11 +173,12 @@ protected :
     void MCALabelSubDescriptor();
     void TimedTextDescriptor();
     void TimedTextResourceSubDescriptor();
-    void Unknown67SubDescriptor();
+    void ContainerConstraintsSubDescriptor();
     void Mpeg4VisualSubDescriptor();
     void AudioChannelLabelSubDescriptor();
     void SoundfieldGroupLabelSubDescriptor();
     void GroupOfSoundfieldGroupsLabelSubDescriptor();
+    void AVCSubDescriptor();
     void OpenIncompleteHeaderPartition();
     void ClosedIncompleteHeaderPartition();
     void OpenCompleteHeaderPartition();
@@ -331,7 +333,7 @@ protected :
     void JPEG2000PictureSubDescriptor_CodingStyleDefault();     //
     void JPEG2000PictureSubDescriptor_QuantizationDefault();    //
     void MpegAudioDescriptor_BitRate();                         //
-    void MultipleDescriptor_SubDescriptorUIDs();                //3F01
+    void MultipleDescriptor_FileDescriptors();                  //3F01
     void PrimaryExtendedSpokenLanguage();                       //
     void SecondaryExtendedSpokenLanguage();                     //
     void OriginalExtendedSpokenLanguage();                      //
@@ -357,6 +359,21 @@ protected :
     void Mpeg4VisualDescriptor_BPictureCount() {MPEG2VideoDescriptor_BPictureCount();}
     void Mpeg4VisualDescriptor_ProfileAndLevel();               //
     void Mpeg4VisualDescriptor_BitRate() {MPEG2VideoDescriptor_BitRate();}
+    void AVCDescriptor_ConstantBFrames() {MPEG2VideoDescriptor_ConstantBFrames();}
+    void AVCDescriptor_CodedContentType() {MPEG2VideoDescriptor_CodedContentType();}
+    void AVCDescriptor_ClosedGOP() {MPEG2VideoDescriptor_ClosedGOP();}
+    void AVCDescriptor_IdenticalGOP() {MPEG2VideoDescriptor_IdenticalGOP();}
+    void AVCDescriptor_MaxGOP() {MPEG2VideoDescriptor_MaxGOP();}
+    void AVCDescriptor_BPictureCount() {MPEG2VideoDescriptor_BPictureCount();}
+    void AVCDescriptor_Profile();
+    void AVCDescriptor_MaximumBitRate();
+    void AVCDescriptor_ProfileConstraint();
+    void AVCDescriptor_Level();
+    void AVCDescriptor_DecodingDelay();
+    void AVCDescriptor_MaximumRefFrames();
+    void AVCDescriptor_SequenceParameterSetFlag();
+    void AVCDescriptor_PictureParameterSetFlag();
+    void AVCDescriptor_AverageBitRate();
     void NetworkLocator_URLString();                            //4001
     void Preface_LastModifiedDate();                            //3B02
     void Preface_ContentStorage();                              //3B03
@@ -607,6 +624,7 @@ protected :
     bool   IsSearchingFooterPartitionAddress;
     bool   FooterPartitionAddress_Jumped;
     bool   PartitionPack_Parsed;
+    bool   HeaderPartition_IsOpen;
     size_t IdIsAlwaysSame_Offset;
 
     //Primer
@@ -715,6 +733,9 @@ protected :
         int64u      Field_Count_InThisBlock_1;
         int64u      Field_Count_InThisBlock_2;
         int64u      Frame_Count_NotParsedIncluded;
+        #if MEDIAINFO_TRACE
+            int64u  Trace_Count;
+        #endif // MEDIAINFO_TRACE
         frame_info  FrameInfo;
 
         essence()
@@ -732,6 +753,9 @@ protected :
             Field_Count_InThisBlock_1=0;
             Field_Count_InThisBlock_2=0;
             Frame_Count_NotParsedIncluded=(int64u)-1;
+            #if MEDIAINFO_TRACE
+                Trace_Count=0;
+            #endif // MEDIAINFO_TRACE
             FrameInfo.DTS=(int64u)-1;
         }
 
@@ -1125,7 +1149,7 @@ protected :
         inline void Locators_Test() {}
     #endif //defined(MEDIAINFO_REFERENCES_YES)
     void NextRandomIndexPack();
-    void TryToFinish();
+    bool BookMark_Needed();
 
     //Temp
     int128u EssenceContainer_FromPartitionMetadata;
@@ -1141,6 +1165,11 @@ protected :
     int64u SDTI_SizePerFrame;
     bool   SDTI_IsPresent; //Used to test if SDTI packet is used for Index StreamOffset calculation
     bool   SDTI_IsInIndexStreamOffset; //Used to test if SDTI packet is used for Index StreamOffset calculation
+    #if MEDIAINFO_TRACE
+        int64u SDTI_SystemMetadataPack_Trace_Count;
+        int64u SDTI_PackageMetadataSet_Trace_Count;
+        int64u Padding_Trace_Count;
+    #endif // MEDIAINFO_TRACE
     string SystemScheme1_TimeCodeArray_StartTimecode;
     int64u SystemScheme1_TimeCodeArray_StartTimecode_ms;
     int64u SystemScheme1_FrameRateFromDescriptor;
@@ -1172,6 +1201,7 @@ protected :
 
     //Hints
     size_t* File_Buffer_Size_Hint_Pointer;
+    size_t  Synched_Count;
 
     //Partitions
     struct partition
