@@ -40,9 +40,9 @@
 #if defined(MEDIAINFO_MPEGA_YES)
     #include "MediaInfo/Audio/File_Mpega.h"
 #endif
+#include "MediaInfo/MediaInfo_Config_MediaInfo.h"
 #if MEDIAINFO_DEMUX
-    #include "MediaInfo/MediaInfo_Config_MediaInfo.h"
-    #include "base64.h"
+    #include "ThirdParty/base64/base64.h"
 #endif //MEDIAINFO_DEMUX
 #include "ZenLib/Utils.h"
 using namespace ZenLib;
@@ -352,10 +352,14 @@ void File_Wm::Header_StreamProperties_Audio ()
     CodecID_Fill(Codec, Stream_Audio, StreamPos_Last, InfoCodecID_Format_Riff);
     Fill(Stream_Audio, StreamPos_Last, Audio_Codec, Codec); //May be replaced by codec parser
     Fill(Stream_Audio, StreamPos_Last, Audio_Codec_CC, Codec);
-    Fill(Stream_Audio, StreamPos_Last, Audio_Channel_s_, Channels);
-    Fill(Stream_Audio, StreamPos_Last, Audio_SamplingRate, SamplingRate);
-    Fill(Stream_Audio, StreamPos_Last, Audio_BitRate, BytesPerSec*8);
-    Fill(Stream_Audio, StreamPos_Last, Audio_BitDepth, Resolution);
+    if (Channels)
+        Fill(Stream_Audio, StreamPos_Last, Audio_Channel_s_, Channels);
+    if (SamplingRate)
+        Fill(Stream_Audio, StreamPos_Last, Audio_SamplingRate, SamplingRate);
+    if (BytesPerSec)
+        Fill(Stream_Audio, StreamPos_Last, Audio_BitRate, BytesPerSec*8);
+    if (Resolution)
+        Fill(Stream_Audio, StreamPos_Last, Audio_BitDepth, Resolution);
 
     FILLING_BEGIN();
         //Creating the parser
@@ -408,7 +412,7 @@ void File_Wm::Header_StreamProperties_Audio_WMA ()
                                 std::string Data_Raw((const char*)(Buffer+Buffer_Offset+Element_Offset), (size_t)10);//Element_Size-(Element_Offset));
                                 std::string Data_Base64(Base64::encode(Data_Raw));
                                 Fill(Stream_Audio, StreamPos_Last, "Demux_InitBytes", Data_Base64);
-                                (*Stream_More)[Stream_Audio][StreamPos_Last](Ztring().From_Local("Demux_InitBytes"), Info_Options)=__T("N NT");
+                                Fill_SetOptions(Stream_Audio, StreamPos_Last, "Demux_InitBytes", "N NT");
                                 }
                                 break;
                     default :   ;
@@ -517,7 +521,7 @@ void File_Wm::Header_StreamProperties_Video ()
                                 std::string Data_Raw((const char*)(Buffer+(size_t)Element_Offset), (size_t)(Data_Size-40));
                                 std::string Data_Base64(Base64::encode(Data_Raw));
                                 Fill(Stream_Video, StreamPos_Last, "Demux_InitBytes", Data_Base64);
-                                (*Stream_More)[Stream_Video][StreamPos_Last](Ztring().From_Local("Demux_InitBytes"), Info_Options)=__T("N NT");
+                                Fill_SetOptions(Stream_Video, StreamPos_Last, "Demux_InitBytes", "N NT");
                                 }
                                 break;
                     default :   ;
@@ -1572,7 +1576,7 @@ void File_Wm::Data_Packet()
 
                 Open_Buffer_Continue(StreamItem.Parser, (size_t)PayloadLength);
                 if (StreamItem.Parser->Status[IsFinished]
-                 || (StreamItem.PresentationTimes.size()>=300 && MediaInfoLib::Config.ParseSpeed_Get()<1))
+                 || (StreamItem.PresentationTimes.size()>=300 && Config->ParseSpeed<1.0))
                 {
                     StreamItem.Parser->Open_Buffer_Unsynch();
                     StreamItem.SearchingPayload=false;
@@ -1599,7 +1603,7 @@ void File_Wm::Data_Packet()
         Skip_XX(Data_Parse_Padding,                             "Padding");
 
     //Jumping if needed
-    if (Streams_Count==0 || (Packet_Count>=1000 && MediaInfoLib::Config.ParseSpeed_Get()<1))
+    if (Streams_Count==0 || (Packet_Count>=1000 && Config->ParseSpeed<1.0))
     {
         Info("Data, Jumping to end of chunk");
         GoTo(Data_AfterTheDataChunk, "Windows Media");
