@@ -52,7 +52,10 @@ void Reader_File_Thread::Entry()
 
     for (;;)
     {
-        Base->CS.Enter();
+        size_t ToRead;
+        size_t Buffer_ToReadOffset;
+        {
+        CriticalSectionLocker CSL(Base->CS);
         if (Base->Buffer_Begin==Base->Buffer_Max)
         {
             Base->IsLooping=false;
@@ -61,8 +64,6 @@ void Reader_File_Thread::Entry()
             Base->Buffer_Begin=0;
         }
 
-        size_t ToRead;
-        size_t Buffer_ToReadOffset;
         if (Base->IsLooping)
         {
             ToRead=Base->Buffer_Begin-Base->Buffer_End2;
@@ -73,8 +74,7 @@ void Reader_File_Thread::Entry()
             ToRead=Base->Buffer_Max-Base->Buffer_End;
             Buffer_ToReadOffset=Base->Buffer_End;
         }
-        Base->CS.Leave();
-
+		}
         if (ToRead)
         {
             if (ToRead>ReadSize_Max)
@@ -83,7 +83,8 @@ void Reader_File_Thread::Entry()
             if (!BytesRead)
                 break;
 
-            Base->CS.Enter();
+            {
+            CriticalSectionLocker CSL(Base->CS);
             if (Base->IsLooping)
             {
                 Base->Buffer_End2+=BytesRead;
@@ -96,8 +97,7 @@ void Reader_File_Thread::Entry()
                     Base->IsLooping=true;
                 }
             }
-            Base->CS.Leave();
-
+			}
             #ifdef WINDOWS
                 SetEvent(Base->Condition_WaitingForMoreData);
             #endif //WINDOWS
@@ -186,15 +186,11 @@ size_t Reader_File::Format_Test(MediaInfo_Internal* MI, String File_Name)
         Format=FormatList.begin();
         while (Format!=FormatList.end())
         {
-            const Ztring &Extensions=FormatList.Get(Format->first, InfoFormat_Extensions);
-            if (Extensions.find(Extension)!=Error)
-            {
-                if(Extension.size()==Extensions.size())
-                    break; //Only one extenion in the list
-                if(Extensions.find(Extension+__T(" "))!=Error
-                || Extensions.find(__T(" ")+Extension)!=Error)
-                    break;
-            }
+            ZtringList Extensions;
+            Extensions.Separator_Set(0, __T(" "));
+            Extensions.Write(FormatList.Get(Format->first, InfoFormat_Extensions));
+            if (Extensions.Find(Extension)!=Error)
+                break;
             ++Format;
         }
     }
@@ -235,6 +231,7 @@ size_t Reader_File::Format_Test_PerParser(MediaInfo_Internal* MI, const String &
     MI->Config.File_Current_Size=MI->Config.File_Size;
     MI->Config.File_Sizes.clear();
     MI->Config.File_Sizes.push_back(MI->Config.File_Size);
+    MI->Config.File_Names_Pos=1;
     if (MI->Config.File_Names.size()>1)
     {
         #if MEDIAINFO_ADVANCED
@@ -281,6 +278,7 @@ size_t Reader_File::Format_Test_PerParser(MediaInfo_Internal* MI, const String &
         Partial_Begin=0; //Wrong value
     if (Partial_Begin>Partial_End)
         Partial_Begin=0; //Wrong value
+    CountOfSeconds=0;
 
     //Parser
     MI->Open_Buffer_Init((Partial_End<=MI->Config.File_Size?Partial_End:MI->Config.File_Size)-Partial_Begin, File_Name);
@@ -338,7 +336,7 @@ size_t Reader_File::Format_Test_PerParser_Continue (MediaInfo_Internal* MI)
         #endif //MEDIAINFO_DEMUX
 
         //Threading
-        if (MI->IsTerminating())
+        if (MI->IsTerminating() || MI->Config.RequestTerminate)
             return 1; //Termination is requested
 
         if (Status[File__Analyze::IsFinished] || (StopAfterFilled && Status[File__Analyze::IsFilled]))
@@ -358,10 +356,10 @@ size_t Reader_File::Format_Test_PerParser_Continue (MediaInfo_Internal* MI)
             return 2; //Must return immediately
 
         //Threading
-        if (MI->IsTerminating())
+        if (MI->IsTerminating() || MI->Config.RequestTerminate)
             return 1; //Termination is requested
 
-        if (Status[File__Analyze::IsFinished] || (StopAfterFilled && Status[File__Analyze::IsFilled]))
+        if (Status[File__Analyze::IsFinished] || MI->Config.IsFinishing || (StopAfterFilled && Status[File__Analyze::IsFilled]))
             ShouldContinue=false;
     }
     #endif //MEDIAINFO_DEMUX
@@ -510,6 +508,8 @@ size_t Reader_File::Format_Test_PerParser_Continue (MediaInfo_Internal* MI)
                     MI->Config.File_Buffer_Size_Max=1;
                 while (MI->Config.File_Buffer_Size_ToRead>MI->Config.File_Buffer_Size_Max)
                     MI->Config.File_Buffer_Size_Max*=2;
+                if (MI->Config.File_Buffer_Size_Max>=64*1024*1024)
+                    MI->Config.File_Buffer_Size_Max=64*1024*1024; //limitation of the buffer in order to avoid to big memory usage
                 MI->Config.File_Buffer=new int8u[MI->Config.File_Buffer_Size_Max];
             }
 
@@ -533,11 +533,18 @@ size_t Reader_File::Format_Test_PerParser_Continue (MediaInfo_Internal* MI)
                     F.Open(MI->Config.File_Names[MI->Config.File_Names_Pos]);
                     while (!F.Opened_Get())
                     {
+                        #if MEDIAINFO_EVENTS
+                            MI->Config.Event_SubFile_Missing_Absolute(MI->Config.File_Names[MI->Config.File_Names_Pos]);
+                        #endif //MEDIAINFO_EVENTS
                         if (MI->Config.File_Names_Pos+1<MI->Config.File_Names.size())
                         {
                             MI->Config.File_Names_Pos++;
                             F.Open(MI->Config.File_Names[MI->Config.File_Names_Pos]);
                         }
+                        else //break the otherwise infinite loop
+                        {
+                            break;
+                        }
                     }
                     if (MI->Config.File_Names_Pos>=MI->Config.File_Sizes.size())
                     {
@@ -626,60 +633,118 @@ size_t Reader_File::Format_Test_PerParser_Continue (MediaInfo_Internal* MI)
             */
 
             //Testing growing files
-            int64u Growing_Temp=(int64u)-1;
             if (MI->Config.ParseSpeed>=1.0 && !MI->Config.File_IsGrowing && MI->Config.File_Current_Offset+F.Position_Get()>=MI->Config.File_Size)
             {
+                if (MI->Config.File_TestContinuousFileNames_Get())
+                {
+                    //int64u Growing_Temp=MI->Config.File_Names.size();
+                    if (MI->Config.File_Names.size()>=24) // only if already a sequence of files
+                        MI->TestContinuousFileNames();
+                    /* TODO: fix about sequences of files
+                    if (MI->Config.File_Names.size()!=Growing_Temp)
+                        MI->Config.File_IsGrowing=true;
+                    */
+                }
                 if (MI->Config.File_Names.size()==1)
                 {
-                    Growing_Temp=F.Size_Get();
+                    int64u Growing_Temp=F.Size_Get();
                     if (MI->Config.File_Size!=Growing_Temp)
                         MI->Config.File_IsGrowing=true;
                 }
-                else if (MI->Config.File_TestContinuousFileNames_Get())
-                {
-                    Growing_Temp=MI->Config.File_Names.size();
-                    MI->TestContinuousFileNames();
-                    if (MI->Config.File_Names.size()!=Growing_Temp)
-                        MI->Config.File_IsGrowing=true;
-                }
             }
             if (MI->Config.File_IsNotGrowingAnymore)
             {
                 MI->Config.File_Current_Size=MI->Config.File_Size=F.Size_Get();
                 MI->Open_Buffer_Init(MI->Config.File_Size, F.Position_Get()-MI->Config.File_Buffer_Size);
                 MI->Config.File_IsGrowing=false;
-                MI->Config.File_IsNotGrowingAnymore=false;
             }
-            if (MI->Config.File_IsGrowing && (Growing_Temp!=(int64u)-1 || MI->Config.File_Current_Offset+F.Position_Get()>=MI->Config.File_Size))
+            if ((MI->Config.File_IsGrowing                                                                          //Was previously detected as growing
+              || (!MI->Config.File_IsNotGrowingAnymore && MI->Config.File_GrowingFile_Force_Get()))                 //Forced test and container did not indicate it is not growing anymore
+                #if MEDIAINFO_READTHREAD
+                    && (!ThreadInstance || (!IsLooping && Buffer_Begin+MI->Config.File_Buffer_Size>=Buffer_End))    //File buffer hit the end of buffer
+                #endif //MEDIAINFO_READTHREAD
+             && MI->Config.File_Current_Offset+F.Position_Get()>=MI->Config.File_Size                               //File read hit the end of file
+             && MI->Config.File_Names.size()==1) //TODO: fix about sequences of files
             {
-                for (size_t CountOfSeconds=0; CountOfSeconds<(size_t)MI->Config.File_GrowingFile_Delay_Get(); CountOfSeconds++)
+                #if MEDIAINFO_EVENTS
+                    {
+                        struct MediaInfo_Event_General_WaitForMoreData_Start_0 Event;
+                        memset(&Event, 0xFF, sizeof(struct MediaInfo_Event_Generic));
+                        Event.EventCode=MediaInfo_EventCode_Create(MediaInfo_Parser_None, MediaInfo_Event_General_WaitForMoreData_Start, 0);
+                        Event.EventSize=sizeof(struct MediaInfo_Event_General_WaitForMoreData_Start_0);
+                        Event.StreamIDs_Size=0;
+                        Event.Duration_Max=(double)MI->Config.File_GrowingFile_Delay_Get();
+                        MI->Config.Event_Send(NULL, (const int8u*)&Event, sizeof(MediaInfo_Event_General_WaitForMoreData_Start_0));
+                    }
+                #endif //MEDIAINFO_EVENTS
+
+                for (; CountOfSeconds<(size_t)MI->Config.File_GrowingFile_Delay_Get(); CountOfSeconds++)
                 {
-                    if (MI->Config.File_Names.size()==1)
+                    int64u LastFile_Size_Old=MI->Config.File_Sizes[MI->Config.File_Sizes.size()-1];
+                    size_t Files_Count_Old=MI->Config.File_Names.size();
+                    //MI->TestContinuousFileNames(); //TODO: fix about sequences of files, "MI->Config.File_Names.size()==1 && " was added "else if (MI->Config.File_TestContinuousFileNames_Get())" commented
+                    int64u LastFile_Size_New=F.Size_Get();
+                    size_t Files_Count_New=MI->Config.File_Names.size();
+                    MI->Open_Buffer_CheckFileModifications();
+
+                    if (LastFile_Size_New != LastFile_Size_Old || Files_Count_New != Files_Count_Old || MI->Config.File_IsNotGrowingAnymore)
                     {
-                        Growing_Temp=F.Size_Get();
-                        if (MI->Config.File_Size!=Growing_Temp)
-                        {
-                            MI->Config.File_Current_Size=MI->Config.File_Size=Growing_Temp;
+                        #if MEDIAINFO_EVENTS
+                            {
+                                struct MediaInfo_Event_General_WaitForMoreData_End_0 Event;
+                                memset(&Event, 0xFF, sizeof(struct MediaInfo_Event_Generic));
+                                Event.EventCode=MediaInfo_EventCode_Create(MediaInfo_Parser_None, MediaInfo_Event_General_WaitForMoreData_End, 0);
+                                Event.EventSize=sizeof(struct MediaInfo_Event_General_WaitForMoreData_End_0);
+                                Event.StreamIDs_Size=0;
+                                Event.Duration_Max=(double)MI->Config.File_GrowingFile_Delay_Get();
+                                Event.Duration_Actual=(double)CountOfSeconds;
+                                Event.Flags=0; //Countinuing
+                                MI->Config.Event_Send(NULL, (const int8u*)&Event, sizeof(MediaInfo_Event_General_WaitForMoreData_End_0));
+                            }
+                        #endif //MEDIAINFO_EVENTS
+                        CountOfSeconds=0;
+                        MI->Config.File_Current_Size=MI->Config.File_Size=LastFile_Size_New; //TODO: check if it is not doable in Open_Buffer_Init() also when MI->Config.File_Names.size() > 1
+                        if (!MI->Config.File_Sizes.empty())
+                            MI->Config.File_Sizes[MI->Config.File_Sizes.size()-1]=LastFile_Size_New;
+                        if (MI->Config.File_Names.size()==1) //if more than 1 file, file size config is already done in TestContinuousFileNames()
                             MI->Open_Buffer_Init(MI->Config.File_Size, MI->Config.File_Current_Offset+F.Position_Get()-MI->Config.File_Buffer_Size);
-                            break;
-                        }
-                    }
-                    else
-                    {
-                        Growing_Temp=MI->Config.File_Names.size();
-                        MI->TestContinuousFileNames();
-                        if (MI->Config.File_Names.size()!=Growing_Temp)
-                        {
-                            MI->Open_Buffer_Init(MI->Config.File_Size, MI->Config.File_Current_Offset+F.Position_Get()-MI->Config.File_Buffer_Size);
-                            break;
-                        }
+                        #if MEDIAINFO_READTHREAD
+                            if (ThreadInstance)
+                                ThreadInstance->RunAgain();
+                        #endif //MEDIAINFO_READTHREAD
+                        break;
                     }
+
                     #ifdef WINDOWS
                         Sleep(1000);
                     #endif //WINDOWS
                 }
+
+                if (CountOfSeconds>=(size_t)MI->Config.File_GrowingFile_Delay_Get())
+                {
+                    #if MEDIAINFO_EVENTS
+                        {
+                            struct MediaInfo_Event_General_WaitForMoreData_End_0 Event;
+                            memset(&Event, 0xFF, sizeof(struct MediaInfo_Event_Generic));
+                            Event.EventCode=MediaInfo_EventCode_Create(MediaInfo_Parser_None, MediaInfo_Event_General_WaitForMoreData_End, 0);
+                            Event.EventSize=sizeof(struct MediaInfo_Event_General_WaitForMoreData_End_0);
+                            Event.StreamIDs_Size=0;
+                            Event.Duration_Max=(double)MI->Config.File_GrowingFile_Delay_Get();
+                            Event.Duration_Actual=(double)CountOfSeconds;
+                            Event.Flags=1; //Giving up
+                            MI->Config.Event_Send(NULL, (const int8u*)&Event, sizeof(MediaInfo_Event_General_WaitForMoreData_End_0));
+                        }
+                    #endif //MEDIAINFO_EVENTS
+
+                    MI->Config.File_IsGrowing=false;
+                }
             }
 
+            if (!MI->Config.File_Buffer_Size
+             && (MI->Config.File_Current_Offset + F.Position_Get()>=MI->Config.File_Size
+              || (MI->Config.File_Size==(int64u)-1 && MI->Config.File_Names_Pos>=MI->Config.File_Names.size() && F.Position_Get()>=F.Size_Get())))
+                break; //Finished, and no other data
+
             #ifdef MEDIAINFO_DEBUG
                 Reader_File_BytesRead_Total+=MI->Config.File_Buffer_Size;
                 Reader_File_BytesRead+=MI->Config.File_Buffer_Size;
@@ -705,7 +770,7 @@ size_t Reader_File::Format_Test_PerParser_Continue (MediaInfo_Internal* MI)
                 }
             #endif //MEDIAINFO_READTHREAD
 
-            if (MI->Config.File_Buffer_Size==0)
+            if (!MI->Config.File_IsGrowing && MI->Config.File_Buffer_Size==0)
             {
                 #if MEDIAINFO_EVENTS
                     MediaInfoLib::Config.Log_Send(0xC0, 0xFF, 0xF0F00101, "File read error");
@@ -719,7 +784,7 @@ size_t Reader_File::Format_Test_PerParser_Continue (MediaInfo_Internal* MI)
             #endif //MEDIAINFO_DEMUX
 
             //Threading
-            if (MI->IsTerminating())
+            if (MI->IsTerminating() || MI->Config.RequestTerminate)
                 break; //Termination is requested
         }
     }
